// Contains code adapted from softSurfer
// Copyright 2001, softSurfer (www.softsurfer.com)
// This code may be freely used and modified for any purpose
// providing that this copyright notice is included with it.
// SoftSurfer makes no warranty for this code, and cannot be held
// liable for any real or imagined damage resulting from its use.
// Users of this code must verify correctness for their application.

#include "ldraw.hpp"


// Class Point
	Point::Point()
	{
		//printf("constructeur point\n");
		m_x=0;
		m_y=0;
		m_z=0;
	}

	Point::Point(const Point &p)
	{
		m_x=p.m_x;
		m_y=p.m_y;
		m_z=p.m_z;
	}

	Point::Point(double x, double y, double z )
	{
		m_x=x;
		m_y=y;
		m_z=z;
	}

	Point::addp(Point* p)
	{
		p->m_x += m_x;
		p->m_y += m_y;
		p->m_z += m_z;
	}

	void Point::setp (double xi, double yi, double zi)
	{
		m_x=xi;
		m_y=yi;
		m_z=zi;
	}
	void Point::setp (const Point &p)
	{
		m_x=p.m_x;
		m_y=p.m_y;
		m_z=p.m_z;
	}

	char* Point::pparse (char* str)
	{
		char *p;
		p=str;
		if(sscanf(p,"%lf", &m_x) == 0) return NULL;
		// eat front spaces
		while(isspace(*p)!=0) p++;
		// find next space
		while(isspace(*p)==0) p++;
		if(sscanf(p,"%lf", &m_y) == 0) return NULL;
		// eat front spaces
		while(isspace(*p)!=0) p++;
		// find next space
		while(isspace(*p)==0) p++;
		if(sscanf(p,"%lf", &m_z) == 0) return NULL;
		// eat front spaces
		while(isspace(*p)!=0) p++;
		// find next space
		while(isspace(*p)==0) p++;
		return p;
	}

	double Point::distance (Point &p)
	{
		// Euclidian distance
		// return sqrt((p.m_x-m_x)*(p.m_x-m_x) + (p.m_y-m_y)*(p.m_y-m_y) + (p.m_z-m_z)*(p.m_z-m_z));
		// Manhattan distance
		return fabs(p.m_x-m_x) + fabs(p.m_y-m_y) + fabs(p.m_z-m_z);
	}

	// returns true if two points are close enough
	bool Point::compare(Point &p)
	{
		CompPoint++;
		if(fabs(p.m_x-m_x)>LdPrecision) return false;
		if(fabs(p.m_y-m_y)>LdPrecision) return false;
		if(fabs(p.m_z-m_z)>LdPrecision) return false;
		return true;
	}
	
	string Point::toString()
	{
		char buf[100];
		sprintf(buf,"%g %g %g", m_x, m_y, m_z);
		return buf;
	}

	void Point::print(bool lf) // parametre par defaut
	{

		cout << "x " << m_x << " y " << m_y << " z " << m_z;
// Version utilisant la surcharge. this est un pointeur -> on utilise *this
//		cout << *this;
//		if(lf) cout << endl;
	}
	void Point::printclassname()
	{
		cout << "classe point" << endl;
	}

	
// Surcharge de << pour pouvoir imprimer des points directement.
/*
ostream& operator << (ostream &s, Point &p)
{
	return s << p.m_x << ", " << p.m_y << ", " << p.m_z;
}

*/
//  == overload to compare "close enough" points
bool operator == (Point &p1, Point &p2)
{
	return (p1.compare(p2));
//	return (p1.distance(p2)<LdPrecision);
}

// Class Vector
/*
Vector::Vector(Point &p)
{
	m_x = p.getx();
	m_y = p.gety();
	m_z = p.getz();
}

Vector::Vector(double x, double y, double z )
{
	m_x = x;
	m_y = y;
	m_z = z;
}
*/
Vector::Vector(Point &p1, Point &p2)
{
	m_x = p2.getx() - p1.getx();
	m_y = p2.gety() - p1.gety();
	m_z = p2.getz() - p1.getz();
}

Vector* Vector::cross (Vector &v)
{
	double x,y,z;
	x = m_y*v.m_z - m_z*v.m_y;
	y = m_z*v.m_x - m_x*v.m_z;
	z = m_x*v.m_y - m_y*v.m_x;
	return new Vector(x,y,z);
}

double Vector::dot (Vector &v)
{
	return ( m_x*v.m_x + m_y*v.m_y + m_z*v.m_z );
}

double Vector::norm ()
{
	return sqrt( m_x*m_x + m_y*m_y + m_z*m_z );
}

void Vector::normalize ()
{
	double length;
	length=norm();
	if (length < EPSILON) return;
	m_x /= length;
	m_y /= length;
	m_z /= length;
}


void Vector::mult(double k)
{
	m_x=m_x*k;
	m_y=m_y*k;
	m_z=m_z*k;
}

// Class Matrix

Matrix::Matrix()
{
	m_mat[0][0]=0;
	m_mat[0][1]=0;
	m_mat[0][2]=0;
	m_mat[1][0]=0;
	m_mat[1][1]=0;
	m_mat[1][2]=0;
	m_mat[2][0]=0;
	m_mat[2][1]=0;
	m_mat[2][2]=0;
}

Matrix::Matrix(double a,double b,double c,double d,double e,double f,double g,double h,double i)
{
	m_mat[0][0]=a;
	m_mat[0][1]=b;
	m_mat[0][2]=c;
	m_mat[1][0]=d;
	m_mat[1][1]=e;
	m_mat[1][2]=f;
	m_mat[2][0]=g;
	m_mat[2][1]=h;
	m_mat[2][2]=i;
}

Matrix::Matrix(Point &p1, Point &p2, Point &p3)
{
	m_mat[0][0]=p1.getx();
	m_mat[0][1]=p1.gety();
	m_mat[0][2]=p1.getz();
	m_mat[1][0]=p2.getx();
	m_mat[1][1]=p2.gety();
	m_mat[1][2]=p2.getz();
	m_mat[2][0]=p3.getx();
	m_mat[2][1]=p3.gety();
	m_mat[2][2]=p3.getz();
}

string Matrix::toString()
{
	char buf[100];
	sprintf(buf,"%g %g %g %g %g %g %g %g %g", m_mat[0][0], m_mat[0][1], m_mat[0][2], m_mat[1][0], m_mat[1][1], m_mat[1][2], m_mat[2][0], m_mat[2][1], m_mat[2][2]);
	return buf;
}

Vector* Matrix::Mult(Vector &v1)
{
	double x,y,z;
	x =  m_mat[0][0]*v1.getx() +  m_mat[0][1]*v1.gety() + m_mat[0][2]*v1.getz();
	y =  m_mat[1][0]*v1.getx() +  m_mat[1][1]*v1.gety() + m_mat[1][2]*v1.getz();
	z =  m_mat[2][0]*v1.getx() +  m_mat[2][1]*v1.gety() + m_mat[2][2]*v1.getz();
	return new Vector(x,y,z);
}

Matrix* Matrix::MatMult(Matrix &m1)
{
	Matrix* m=new Matrix(0,0,0,0,0,0,0,0,0);
	for(int i=0; i<3; i++)
	{
		for(int j=0; j<3; j++)
		{
			for(int k=0; k<3; k++)
			{
				m->m_mat[i][j] += m_mat[i][k] * m1.m_mat[k][j];
			}
		}
	}
	return m;
}

double Matrix::MatDeterminant()
{
	return m_mat[0][0]*m_mat[1][1]*m_mat[2][2] + m_mat[1][0]*m_mat[2][1]*m_mat[0][2] + m_mat[2][0]*m_mat[0][1]*m_mat[1][2]
		- m_mat[2][0]*m_mat[1][1]*m_mat[0][2] - m_mat[0][0]*m_mat[2][1]*m_mat[1][2] - m_mat[1][0]*m_mat[0][1]*m_mat[2][2];
}


// Class LDComment

LdComment::LdComment(const LdComment &ldc)
{
	m_cs=ldc.m_cs;
}

LdComment::LdComment(const string &str)
{
	m_cs=str;
}

LdComment::LdComment(const char *str)
{
	m_cs=str;
}

string LdComment::toString()
{
	string result = "0 ";
	result += m_cs;
	return result;
}

int LdComment::readbfc()
{
	char comment[1000];
	strcpy(comment, m_cs.c_str());
	for(int i=0; i<strlen(comment); i++) comment[i]=toupper(comment[i]);
	if(strstr(comment,"BFC") == NULL) return 0;
	if(strstr(comment,"INVERTNEXT") != NULL) return 1;
	if(strstr(comment,"CERTIFY") == NULL) return 0;
	if(strstr(comment,"CCW") != NULL) return 2;
	if(strstr(comment,"CW") != NULL) return 3;
	return 2;
}

// Class LDSubfile
// *** treated as comment !!!***

/*
LdSubfile::LdSubfile(const LdSubfile &ldc)
{
	m_cs=ldc.m_cs;
}


LdSubfile::LdSubfile(const string &str)
{
	m_cs=str;
}
*/

LdSubfile::LdSubfile(const char *str)
{
	char*p;
	Point p1,p2,p3;
	p=(char*)str;
	// eat front spaces
	while(isspace(*p)!=0) p++;
	// find next space after type
	while(isspace(*p)==0) p++;
	sscanf(p,"%d", &m_col);
	// skip color
	while(isspace(*p)!=0) p++;
	// find next space
	while(isspace(*p)==0) p++;
	// Read 1st point
	p=m_origin.pparse(p);
	p=p1.pparse(p);
	p=p2.pparse(p);
	p=p3.pparse(p);
	m_mat=Matrix(p1,p2,p3);
	while(isspace(*p)!=0) p++;
	m_file=p;
}

LdSubfile::translate(Point &p)
{
	p.addp(&m_origin);
}

LdSubfile::transform(Matrix &m)
{
	Vector* v1;
	v1=m.Mult((Vector)m_origin);
	m_origin=*v1;
	delete v1;
	Matrix* m2;
	m2=m.MatMult(m_mat);
	m_mat=*m2;
	delete m2;
}


int LdSubfile::checkSide(LdObject *ldo)
{
	int incpt=0, outcpt=0;
	double a;
	Point closest;
	a = ldo->planeDist(m_origin, &closest);
	if(a > LdPrecision) outcpt++;
	else if(a < -LdPrecision) incpt++;
	else m_origin.setp(closest);
	if(incpt>0) return 1;
	else if(outcpt>0) return -1;
	else return 2;
}


string LdSubfile::toString()
{
	char color[20];
	string result = "1 ";
	sprintf(color,"%d ",m_col);
	result += color;
	result += m_origin.toString();
	result += " ";
	result += m_mat.toString();
	result += " ";
	result += m_file;
	return result;
}


// Class LdLine

LdLine::LdLine(const char *str)
{
	char*p;
	p=(char*)str;
	// eat front spaces
	while(isspace(*p)!=0) p++;
	// find next space after type
	while(isspace(*p)==0) p++;
	sscanf(p,"%d", &m_col);
	// skip color
	while(isspace(*p)!=0) p++;
	// find next space
	while(isspace(*p)==0) p++;
	// Read 1st point
	p=m_p1.pparse(p);
	p=m_p2.pparse(p);
}

LdLine::LdLine(const Point &p1, const Point &p2, const int col)
{
	m_p1=p1;
	m_p2=p2;
	m_col=col;
}

string LdLine::toString()
{
	char color[20];
	string result = "2 ";
	sprintf(color,"%d ",m_col);
	result += color;
	result += m_p1.toString();
	result += " ";
	result += m_p2.toString();
	return result;
}

LdLine::translate(Point &p)
{
	p.addp(&m_p1);
	p.addp(&m_p2);
}

LdLine::transform(Matrix &m)
{
	Vector* v1;
	v1=m.Mult((Vector)m_p1);
	m_p1=*v1;
	delete v1;
	v1=m.Mult((Vector)m_p2);
	m_p2=*v1;
	delete v1;
}

LdObject* LdLine::getFirstLine()
{
	return new LdLine(m_p1, m_p2);
}

bool LdLine::matchLine(LdObject &line)
{
/*	if(line.getType() != Line)
	{
		cout << "Trying to use matchline with a non-line" << endl;
		return false;
	}
*/	
	if((m_p1 == ((LdLine&)line).m_p1) && (m_p2 == ((LdLine&)line).m_p2)) return true;
	if((m_p2 == ((LdLine&)line).m_p1) && (m_p1 == ((LdLine&)line).m_p2)) return true;

/* // Following code is not more efficient
	if(m_p1 == ((LdLine&)line).m_p1)
	{
		if(m_p2 == ((LdLine&)line).m_p2) return true;
		return false;
	}
	else if(m_p2 == ((LdLine&)line).m_p1)
	{
		if(m_p1 == ((LdLine&)line).m_p2) return true;
		return false;
	}
*/
	return false;
}

int LdLine::checkSide(LdObject *ldo)
{
	int incpt=0, outcpt=0;
	double a;
	Point closest;
	a = ldo->planeDist(m_p1, &closest);
	if(a > LdPrecision) outcpt++;
	else if(a < -LdPrecision) incpt++;
	else m_p1.setp(closest);
	a = ldo->planeDist(m_p2, &closest);
	if(a > LdPrecision) outcpt++;
	else if(a < -LdPrecision) incpt++;
	else m_p2.setp(closest);
	if(incpt>0)
	{
		if(outcpt==0) return 1;
		else return 0;
	}
	else 
	{
		if(outcpt>0) return -1;
		else return 2;
	}
}

bool LdLine::checkSym (LdObject *ldo)
{
	Point closest;
	if(ldo->checkSym(m_p1, m_p2)) return true;
	return false;
}

void LdLine::Cut(LdObject* surf, LdObject* &o1, LdObject* &o2)
{
	
	Point inter;
	Point p;
	if(surf->segInter(m_p1, m_p2, &inter) != 1) return;
	if(surf->planeDist(m_p1,&p) < 0)
	{
		o1=new LdLine(m_p1, inter, getColor());
	}
	else o1=new LdLine(inter, m_p2, getColor());
}

bool LdLine::matchPoint(Point &p1)
{
	if((m_p1==p1) || (m_p2==p1)) return true;
	return false;
}

// Class LdTriangle

LdTriangle::LdTriangle(const char *str)
{
	char*p;
	p=(char*)str;
	// eat front spaces
	while(isspace(*p)!=0) p++;
	// find next space after type
	while(isspace(*p)==0) p++;
	sscanf(p,"%d", &m_col);
	// skip color
	while(isspace(*p)!=0) p++;
	// find next space
	while(isspace(*p)==0) p++;
	// Read 1st point
	p=m_p1.pparse(p);
	p=m_p2.pparse(p);
	p=m_p3.pparse(p);
}

LdTriangle::LdTriangle(Point &p1, Point &p2, Point &p3, int col)
{
	m_p1=p1;
	m_p2=p2;
	m_p3=p3;
	m_col=col;
}

string LdTriangle::toString()
{
	char color[20];
	string result = "3 ";
	sprintf(color,"%d ",m_col);
	result += color;
	result += m_p1.toString();
	result += " ";
	result += m_p2.toString();
	result += " ";
	result += m_p3.toString();
	return result;
}


LdTriangle::translate(Point &p)
{
	p.addp(&m_p1);
	p.addp(&m_p2);
	p.addp(&m_p3);
}

void LdTriangle::RotPoints(void)
{
	Point temp;
	temp = m_p1;
	m_p1=m_p2;
	m_p2=m_p3;
	m_p3=temp;
}

LdTriangle::transform(Matrix &m)
{
	Vector* v1;
	v1=m.Mult((Vector)m_p1);
	m_p1=*v1;
	delete v1;
	v1=m.Mult((Vector)m_p2);
	m_p2=*v1;
	delete v1;
	v1=m.Mult((Vector)m_p3);
	m_p3=*v1;
	delete v1;
}

LdObject* LdTriangle::getFirstLine()
{
	m_edgecpt=0;
	return new LdLine(m_p1,m_p2);
}

LdObject* LdTriangle::getNextLine()
{
	m_edgecpt++;
	switch (m_edgecpt)
	{
	case 1:
		return new LdLine(m_p2,m_p3);
	case 2:
		return new LdLine(m_p3,m_p1);
	default:
		return NULL;
	}
}

bool LdTriangle::matchLine(LdObject &line)
{
	if(line.matchPoint(m_p1))
	{
		if(line.matchPoint(m_p2)) return true;
		if(line.matchPoint(m_p3)) return true;
		return false;
	}
	if(line.matchPoint(m_p2) && line.matchPoint(m_p3)) return true;
	return false;
}

Vector* LdTriangle::getNormal()
{
	Vector v1(m_p1, m_p2);
	Vector v2(m_p2, m_p3);
	Vector *v3 = v1.cross(v2);
	v3->normalize();
	return v3;
}

double LdTriangle::getAngle(LdObject &surf, LdObject &line )
{
	double result;
	Vector *v1 = getNormal();
	Vector *v2 = surf.getNormal();
	Vector *v3 = v1->cross(*v2);
	result = v3->norm();
	Vector v4(*((LdLine&)line).getP1(), *((LdLine&)line).getP2());
	result = 180*asin(result)/PI;
	if(v4.dot(*v3) > 0) result=-result;
	if(surf.GetBFC() == -1) result=-result;
	delete v1;
	delete v2;
	delete v3;
	if(result > 1-EPSILON) return 90;
	if(result < -1+EPSILON) return -90;
	return 180*asin(result)/PI;
	return result;
}
double LdTriangle::planeDist(Point &p, Point *closest)
{
	Vector *n=getNormal();
	Vector v=Vector(p, m_p1);
	double sn, dist;
	sn=v.dot(*n);
	n->mult(sn);
	Point *pp=(Point*)(n);
	p.addp(pp);
	closest->setp(*pp);
    dist=p.distance(*closest);
	if(sn>0) dist=-dist;
	return dist;
}

bool LdTriangle::checkSym (Point &p1, Point &p2)
{
	Vector *n=getNormal();
	Vector v=Vector(p1, m_p1);
	double sn;
	sn=v.dot(*n);
	n->mult(2*sn);
	Point *pp=(Point*)(n);
	p1.addp(pp);
	if(pp->distance(p2) < LdPrecision) return true;
	else return false;
}

bool LdTriangle::checkSym (LdObject *ldo)
{
	Point closest;
	for(int i=0; i<3; i++)
	{
		if(ldo->planeDist(m_p1, &closest) < LdPrecision && ldo->checkSym(m_p2, m_p3)) return true;
		RotPoints();
	}
	return false;
}



Point* LdTriangle::getOtherPoint (LdObject &line)
{
	if(!((LdLine&)line).matchPoint(m_p1)) return &m_p1;
	if(!((LdLine&)line).matchPoint(m_p2)) return &m_p2;
	return &m_p3;
}

int LdTriangle::checkSide(LdObject *ldo)
{
	int incpt=0, outcpt=0;
	double a;
	Point closest;
	a = ldo->planeDist(m_p1, &closest);
	if(a > LdPrecision) outcpt++;
	else if(a < -LdPrecision) incpt++;
	else m_p1.setp(closest);
	a = ldo->planeDist(m_p2, &closest);
	if(a > LdPrecision) outcpt++;
	else if(a < -LdPrecision) incpt++;
	else m_p2.setp(closest);
	a = ldo->planeDist(m_p3, &closest);
	if(a > LdPrecision) outcpt++;
	else if(a < -LdPrecision) incpt++;
	else m_p3.setp(closest);

	if(incpt>0)
	{
		if(outcpt==0) return 1;
		else return 0;
	}
	else 
	{
		if(outcpt>0) return -1;
		else return 2;
	}

}



/*
// intersect3D_SegmentPlane(): intersect a segment and a plane
//    Input:  S = a segment, and Pn = a plane = {Point V0; Vector n;}
//    Output: *I0 = the intersect point (when it exists)
//    Return: 0 = disjoint (no intersection)
//            1 = intersection in the unique point *I0
//            2 = the segment lies in the plane
int
intersect3D_SegmentPlane( Segment S, Plane Pn, Point* I )
{
    Vector    u = S.P1 - S.P0;
    Vector    w = S.P0 - Pn.V0;

    float     D = dot(Pn.n, u);
    float     N = -dot(Pn.n, w);

    if (fabs(D) < SMALL_NUM) {          // segment is parallel to plane
        if (N == 0)                     // segment lies in plane
            return 2;
        else
            return 0;                   // no intersection
    }
    // they are not parallel
    // compute intersect param
    float sI = N / D;
    if (sI < 0 || sI > 1)
        return 0;                       // no intersection

    *I = S.P0 + sI * u;                 // compute segment intersect point
    return 1;

double LdTriangle::planeDist(Point &p, Point *closest)
{
	Vector *n=getNormal();
	Vector v=Vector(p, m_p1);
	double sn, dist;
	sn=v.dot(*n);
	n->mult(sn);
	Point *pp=(Point*)(n);
	p.addp(pp);
	closest->setp(*pp);
    dist=p.distance(*closest);
	if(sn>0) dist=-dist;
	return dist;
}
*/

int LdTriangle::segInter(Point &p1, Point &p2, Point *intersection)
{
	Vector *n=getNormal(); //surface normal
	Vector u(p2,p1);
	Vector w(p1, m_p1);
	double D = n->dot(u);
	double N = -n->dot(w);
    if (fabs(D) < EPSILON) {	// segment is parallel to plane
        if (N == 0)				// segment lies in plane
            return 2;
        else
            return 0;			// no intersection
    }
	// they are not parallel
    // compute intersect param
    double sI = N / D;
    if (sI < 0 || sI > 1)
        return 0;               // no intersection
	u.mult(-sI);
	Point *p = (Point*)(&u);
	p1.addp(p);
    intersection->setp(*p);		// compute segment intersect point
    return 1;
}

void LdTriangle::Cut(LdObject* surf, LdObject* &o1, LdObject* &o2)
{
	Point p;
	// Find first point on the right side
	while(surf->planeDist(m_p1,&p) < 0) RotPoints();
	while(surf->planeDist(m_p1,&p) >= 0) RotPoints();
	Point p1, p2;
	surf->segInter(m_p3, m_p1, &p1);
	if(surf->planeDist(m_p2,&p) < 0)
	{
		surf->segInter(m_p3, m_p2, &p2);
		o1=new LdQuad(p1, m_p1, m_p2, p2, getColor());
	}
	else
	{
		surf->segInter(m_p2, m_p1, &p2);
		o1=new LdTriangle(p1, m_p1, p2, getColor());
	}
}


void LdTriangle::InvertWinding()
{
	Point p;
	p = m_p1;
	m_p1 = m_p3;
	m_p3 = p;
}


// Class LdQuad

LdQuad::LdQuad(const char *str)
{
	char*p;
	p=(char*)str;
	// eat front spaces
	while(isspace(*p)!=0) p++;
	// find next space after type
	while(isspace(*p)==0) p++;
	sscanf(p,"%d", &m_col);
	// skip color
	while(isspace(*p)!=0) p++;
	// find next space
	while(isspace(*p)==0) p++;
	// Read 1st point
	p=m_p1.pparse(p);
	p=m_p2.pparse(p);
	p=m_p3.pparse(p);
	p=m_p4.pparse(p);
}

LdQuad::LdQuad(Point &p1, Point &p2, Point &p3, Point &p4, int col)
{
	m_p1=p1;
	m_p2=p2;
	m_p3=p3;
	m_p4=p4;
	m_col=col;
}

string LdQuad::toString()
{
	char color[20];
	string result = "4 ";
	sprintf(color,"%d ",m_col);
	result += color;
	result += m_p1.toString();
	result += " ";
	result += m_p2.toString();
	result += " ";
	result += m_p3.toString();
	result += " ";
	result += m_p4.toString();
	return result;
}


LdQuad::translate(Point &p)
{
	p.addp(&m_p1);
	p.addp(&m_p2);
	p.addp(&m_p3);
	p.addp(&m_p4);
}

LdQuad::transform(Matrix &m)
{
	Vector* v1;
	v1=m.Mult((Vector)m_p1);
	m_p1=*v1;
	delete v1;
	v1=m.Mult((Vector)m_p2);
	m_p2=*v1;
	delete v1;
	v1=m.Mult((Vector)m_p3);
	m_p3=*v1;
	delete v1;
	v1=m.Mult((Vector)m_p4);
	m_p4=*v1;
	delete v1;
}

void LdQuad::RotPoints(void)
{
	Point temp;
	temp = m_p1;
	m_p1=m_p2;
	m_p2=m_p3;
	m_p3=m_p4;
	m_p4=temp;
}

LdObject* LdQuad::getFirstLine()
{
	m_edgecpt=0;
	return new LdLine(m_p1,m_p2);
}

LdObject* LdQuad::getNextLine()
{
	m_edgecpt++;
	switch (m_edgecpt)
	{
	case 1:
		return new LdLine(m_p2,m_p3);
	case 2:
		return new LdLine(m_p3,m_p4);
	case 3:
		return new LdLine(m_p4,m_p1);
	default:
		return NULL;
	}
}

bool LdQuad::matchLine(LdObject &line)
{
	if(!line.matchPoint(m_p1) && !line.matchPoint(m_p3)) return false;
	LdLine ltemp (m_p1, m_p2);
	if(ltemp.matchLine(line)) return true;
	ltemp.SetLinePoints(m_p2, m_p3);
	if(ltemp.matchLine(line)) return true;
	ltemp.SetLinePoints(m_p3, m_p4);
	if(ltemp.matchLine(line)) return true;
	ltemp.SetLinePoints(m_p4, m_p1);
	if(ltemp.matchLine(line)) return true;
	return false;
}

Vector* LdQuad::getNormal()
{
	Vector v1(m_p1, m_p2);
	Vector v2(m_p2, m_p3);
	Vector *v3 = v1.cross(v2);
	v3->normalize();
	return v3;
}

double LdQuad::getAngle(LdObject &surf, LdObject &line )
{
	double result;
	Vector *v1 = getNormal();
	Vector *v2 = surf.getNormal();
	Vector *v3 = v1->cross(*v2);
	result = v3->norm();
	Vector v4(*((LdLine&)line).getP1(), *((LdLine&)line).getP2());
	if(v4.dot(*v3) > 0) result=-result;
	if(surf.GetBFC() == -1) result=-result;
	delete v1;
	delete v2;
	delete v3;
	if(result > 1-EPSILON) return 90;
	if(result < -1+EPSILON) return -90;
	return 180*asin(result)/PI;
}

double LdQuad::planeDist(Point &p, Point *closest)
{
	Vector *n=getNormal();
	Vector v=Vector(p, m_p1);
	double sn, dist;
	sn=v.dot(*n);
	n->mult(sn);
	Point *pp=(Point*)(n);
	p.addp(pp);
	closest->setp(*pp);
    dist=p.distance(*closest);
	if(sn>0) dist=-dist;
	return dist;
}

bool LdQuad::checkSym (Point &p1, Point &p2)
{
	Vector *n=getNormal();
	Vector v=Vector(p1, m_p1);
	double sn;
	sn=v.dot(*n);
	n->mult(2*sn);
	Point *pp=(Point*)(n);
	p1.addp(pp);
	if(pp->distance(p2) < LdPrecision) return true;
	else return false;
}


bool LdQuad::checkSym (LdObject *ldo)
{
	Point closest;
	for(int i=0; i<2; i++)
	{
		if(ldo->planeDist(m_p1, &closest) < LdPrecision && ldo->planeDist(m_p3, &closest) < LdPrecision  && ldo->checkSym(m_p2, m_p4)) return true;
		if(ldo->checkSym(m_p1, m_p2) && ldo->checkSym(m_p3, m_p4)) return true;
		RotPoints();
	}
	return false;
}


Point* LdQuad::getOtherPoint (LdObject &line)
{
	if(!((LdLine&)line).matchPoint(m_p1)) return &m_p1;
	if(!((LdLine&)line).matchPoint(m_p2)) return &m_p2;
	return &m_p3;
}

int LdQuad::segInter(Point &p1, Point &p2, Point *intersection)
{
	Vector *n=getNormal(); //surface normal
	Vector u(p2,p1);
	Vector w(p1, m_p1);
	double D = n->dot(u);
	double N = -n->dot(w);
    if (fabs(D) < EPSILON) {	// segment is parallel to plane
        if (N == 0)				// segment lies in plane
            return 2;
        else
            return 0;			// no intersection
    }
	// they are not parallel
    // compute intersect param
    double sI = N / D;
    if (sI < 0 || sI > 1)
        return 0;               // no intersection
	u.mult(-sI);
	Point *p = (Point*)(&u);
	p1.addp(p);
    intersection->setp(*p);		// compute segment intersect point
    return 1;
}

int LdQuad::checkSide(LdObject *ldo)
{
	int incpt=0, outcpt=0;
	double a;
	Point closest;
	a = ldo->planeDist(m_p1, &closest);
	if(a > LdPrecision) outcpt++;
	else if(a < -LdPrecision) incpt++;
	else m_p1.setp(closest);
	a = ldo->planeDist(m_p2, &closest);
	if(a > LdPrecision) outcpt++;
	else if(a < -LdPrecision) incpt++;
	else m_p2.setp(closest);
	a = ldo->planeDist(m_p3, &closest);
	if(a > LdPrecision) outcpt++;
	else if(a < -LdPrecision) incpt++;
	else m_p3.setp(closest);
	a = ldo->planeDist(m_p4, &closest);
	if(a > LdPrecision) outcpt++;
	else if(a < -LdPrecision) incpt++;
	else m_p4.setp(closest);

	if(incpt>0)
	{
		if(outcpt==0) return 1;
		else return 0;
	}
	else 
	{
		if(outcpt>0) return -1;
		else return 2;
	}
}

void LdQuad::Cut(LdObject* surf, LdObject* &o1, LdObject* &o2)
{
	Point p;
	// Find first point on the right side
	while(surf->planeDist(m_p1,&p) < 0) RotPoints();
	while(surf->planeDist(m_p1,&p) >= 0) RotPoints();
	Point p1, p2;
	surf->segInter(m_p4, m_p1, &p1);
	if(surf->planeDist(m_p2,&p) < 0)
	{
		if(surf->planeDist(m_p3,&p) < 0)
		{
			surf->segInter(m_p3, m_p4, &p2);
			o1=new LdQuad(p1, m_p1, m_p2, m_p3, getColor());
			o2=new LdTriangle(m_p3, p2, p1, getColor());
		}
		else
		{
			surf->segInter(m_p3, m_p2, &p2);
			o1=new LdQuad(p1, m_p1, m_p2, p2, getColor());
		}
	}
	else
	{
		surf->segInter(m_p2, m_p1, &p2);
		o1=new LdTriangle(p1, m_p1, p2, getColor());
	}
}



void LdQuad::InvertWinding()
{
	Point p;
	p = m_p1;
	m_p1 = m_p3;
	m_p3 = p;
}



// Class LdCondline

LdCondline::LdCondline(const char *str)
{
	char*p;
	p=(char*)str;
	// eat front spaces
	while(isspace(*p)!=0) p++;
	// find next space after type
	while(isspace(*p)==0) p++;
	sscanf(p,"%d", &m_col);
	// skip color
	while(isspace(*p)!=0) p++;
	// find next space
	while(isspace(*p)==0) p++;
	// Read 1st point
	p=m_p1.pparse(p);
	p=m_p2.pparse(p);
	p=m_p3.pparse(p);
	p=m_p4.pparse(p);
}

LdCondline::LdCondline(Point &p1, Point &p2, Point &p3, Point &p4, const int col)
{
	m_p1=p1;
	m_p2=p2;
	m_p3=p3;
	m_p4=p4;
	m_col=col;
}


string LdCondline::toString()
{
	char color[20];
	string result = "5 ";
	sprintf(color,"%d ",m_col);
	result += color;
	result += m_p1.toString();
	result += " ";
	result += m_p2.toString();
	result += " ";
	result += m_p3.toString();
	result += " ";
	result += m_p4.toString();
	return result;
}


LdCondline::translate(Point &p)
{
	p.addp(&m_p1);
	p.addp(&m_p2);
	p.addp(&m_p3);
	p.addp(&m_p4);
}

LdCondline::transform(Matrix &m)
{
	Vector* v1;
	v1=m.Mult((Vector)m_p1);
	m_p1=*v1;
	delete v1;
	v1=m.Mult((Vector)m_p2);
	m_p2=*v1;
	delete v1;
	v1=m.Mult((Vector)m_p3);
	m_p3=*v1;
	delete v1;
	v1=m.Mult((Vector)m_p4);
	m_p4=*v1;
	delete v1;
}


LdObject* LdCondline::getFirstLine()
{
	return new LdLine(m_p1,m_p2);
}

bool LdCondline::matchLine(LdObject &line)
{
	LdLine ltemp (m_p1, m_p2);
	if(ltemp.matchLine(line)) return true;
	return false;
}

int LdCondline::checkSide(LdObject *ldo)
{
	int incpt=0, outcpt=0;
	double a;
	Point closest;
	a = ldo->planeDist(m_p1, &closest);
	if(a > LdPrecision) outcpt++;
	else if(a < -LdPrecision) incpt++;
	else m_p1.setp(closest);
	a = ldo->planeDist(m_p2, &closest);
	if(a > LdPrecision) outcpt++;
	else if(a < -LdPrecision) incpt++;
	else m_p2.setp(closest);
	if(incpt>0)
	{
		if(outcpt==0) return 1;
		else return 0;
	}
	else 
	{
		if(outcpt>0) return -1;
		else return 2;
	}
}

bool LdCondline::checkSym (LdObject *ldo)
{
	Point closest;
	if(ldo->checkSym(m_p1, m_p2)) return true;
	return false;
}

void LdCondline::Cut(LdObject* surf, LdObject* &o1, LdObject* &o2)
{
	
	Point inter;
	Point p;
	if(surf->segInter(m_p1, m_p2, &inter) != 1) return;
	if(surf->planeDist(m_p1,&p) < 0)
	{
		o1=new LdCondline(m_p1, inter, m_p3, m_p4, getColor());
	}
	else o1=new LdCondline(inter, m_p2, m_p3, m_p4, getColor());
}

// Class LdPath
LdPath::LdPath()
{
/*	string* path;
	path=new string(".\\");
	pathtable.push_back(path);
	path=new string(".\\s\\");
	pathtable.push_back(path);
	path=new string(".\\p\\");
	pathtable.push_back(path);
	ifstream ldrawini("ldraw.ini", ios::in);
	if(ldrawini == NULL) cout << "ldraw.ini not found" << endl;
*/
}

LdPath::~LdPath()
{
	for(int i=0; i<pathtable.size(); i++)
	{
		delete pathtable[i];
	}
}

LdPath::addPath(string &s)
{
	string* path;
	path=new string(s);
	pathtable.push_back(path);
}

bool LdPath::subOpen(ifstream &ldSubfile, string &subfilename)
{
	for(int i=0; i<pathtable.size(); i++)
	{
		string sfn = *pathtable[i];
		sfn += subfilename;
		// cout << sfn.c_str() << endl;
		ldSubfile.open(sfn.c_str(), ios::in);
		if(ldSubfile.good()) return true;
		ldSubfile.clear();
	}
  return false;
}

string LdPath::toString()
{
	string result;
	for(int i=0; i<pathtable.size(); i++)
	{
		result += *pathtable[i];
		if(i==pathtable.size()-1) continue;
		result += ", ";
	}
	return result;
}

// Class Fldraw

#define BUFLEN 1000

void funcdelete (LdObject* ldo) 
{
	delete ldo;
}

Fldraw::~Fldraw()
{
	for_each(ldtab.begin(), ldtab.end(), funcdelete);
}

void Fldraw::InvertWinding()
{
	for(int i=0; i<ldtab.size();i++)
	{
		ldtab[i]->InvertWinding();
	}
}

bool Fldraw::ldLoad (char *fname)
{
	char buf[BUFLEN];
	int type;
	int numline=0;
	int numEdge=0;
	int numCond=0;
	int numTri=0;
	int numQuad=0;
	int numSub=0;
	int curBFC=0;
	int newBFC;
	
	cout << endl << "Loading " << fname << endl;
	// ouverture fichier	
	ifstream ldFileIn(fname, ios::in);
	if(ldFileIn == NULL) return false;

	while (!ldFileIn.eof())
	{
		ldFileIn.getline(buf, BUFLEN);
		if(ldFileIn.fail())
		{
			if(ldFileIn.eof()) break;
			ldFileIn.clear();
			continue;
		}
		if (strlen(buf)==0) continue;
		sscanf(buf,"%d ", &type);

		switch(type)
		{
		case Comment:
			char *p;
			p=strchr(buf,'0');
			if(strlen(p)<2)
				*p=0;
			else
				p+=2;
			ldtab.push_back(new LdComment(p));
			newBFC = ((LdComment*)ldtab.back())->readbfc();
			switch (newBFC)
			{
			case 2:
				curBFC = 1;
				break;
			case 3:
				curBFC = -1;
				break;
			default:
				break;
			}
			break;
		case Subfile:
			numSub++;

			ldtab.push_back(new LdSubfile(buf));
			break;
		case Line:
			numEdge++;
			if(DelEdge) break;
			ldtab.push_back(new LdLine(buf));
			break;
		case Triangle:
			numTri++;
			ldtab.push_back(new LdTriangle(buf));
			break;
		case Quad:
			numQuad++;
			ldtab.push_back(new LdQuad(buf));
			break;
		case Condline:
			numCond++;
			if(DelCond) break;
			ldtab.push_back(new LdCondline(buf));
			break;
		default:
			break;
		}
		if(ldtab.back()->getType() != Subfile)	ldtab.back()->SetBFC(curBFC);
		else ldtab.back()->SetBFC(1);
	}
	m_nLine = ldtab.size();
	cout << m_nLine << " lines in file"<< endl;
	cout << numEdge << " Edge lines";
	if(DelEdge) cout << " (deleted)";
	cout << endl << numCond << " Conditional lines";
	if(DelCond) cout << " (deleted)";
	cout << endl << numTri << " Triangles"<< endl;
	cout << numQuad << " Quads"<< endl;
	cout << numSub << " Subfiles"<< endl;

	return true;
}

Fldraw::ldExpand(LdPath *path)
{
	char buf[BUFLEN];
	int type;
	int numline=0;
	int numEdge=0;
	int numCond=0;
	int numTri=0;
	int numQuad=0;
	int numSub=0;
	int curBFC=0;
	int newBFC;
	int invertBFC;
	int invertsub;
	Point *po;
	Matrix *mat;
	for(int i=0; i<ldtab.size(); i++)
	{
		ifstream ldSubfile;

		if(ldtab[i]->getType() != Subfile) continue;
		// cout << ldtab[i]->toString().c_str() << endl;
		po=((LdSubfile *)ldtab[i])->getOrigin();
		mat=((LdSubfile *)ldtab[i])->getMAtrix();
		if(! path->subOpen(ldSubfile, ((LdSubfile *)ldtab[i])->getName())) continue;
		
		invertBFC = ((LdSubfile *)ldtab[i])->GetBFC();
		if(i>0)
		{
			if((ldtab[i-1]->getType() == Comment) && ((LdComment *)ldtab[i-1])->readbfc() == 1)
				invertBFC *= -1;
		}
		invertsub=invertBFC;
		if(mat->MatDeterminant() < 0) invertBFC *= -1; 
		while (!ldSubfile.eof())
		{
			ldSubfile.getline(buf, BUFLEN);
			if(ldSubfile.fail())
			{
				if(ldSubfile.eof()) break;
				ldSubfile.clear();
				continue;
			}

			if (strlen(buf)==0) continue;
			type=Empty;
			sscanf(buf,"%d ", &type);
			switch(type)
			{
			case Comment:
				char *p;
				p=strchr(buf,'0');
				if(p==NULL) break;
				if(strlen(p)<2)
					*p=0;
				else
					p+=2;
				ldtab.push_back(new LdComment(p));
				newBFC = ((LdComment*)ldtab.back())->readbfc();
				switch (newBFC)
				{
				case 2:
					curBFC = 1;
					break;
				case 3:
					curBFC = -1;
					break;
				case 1:
					break;
				default:
					if(!SubComments)
					{
						delete ldtab.back();
						ldtab.back() = new LdEmpty;
					}
					break;
				}
				break;
			case Subfile:
				numSub++;
				ldtab.push_back(new LdSubfile(buf));
				break;
			case Line:
				numEdge++;
				// if(DelEdge) break;
				ldtab.push_back(new LdLine(buf));
				break;
			case Triangle:
				numTri++;
				ldtab.push_back(new LdTriangle(buf));
				if(invertBFC == -1) ((LdTriangle*)ldtab.back())->InvertWinding();
				break;
			case Quad:
				numQuad++;
				ldtab.push_back(new LdQuad(buf));
				if(invertBFC == -1) ((LdTriangle*)ldtab.back())->InvertWinding();
				break;
			case Condline:
				numCond++;
				// if(DelCond) break;
				ldtab.push_back(new LdCondline(buf));
				break;
			default:
				break;
			}
			if(type != Empty)
			{
				ldtab.back()->SetNoOut(true);
				ldtab.back()->transform(*mat);
				ldtab.back()->translate(*po);
				if(ldtab.back()->getType() != Subfile)	ldtab.back()->SetBFC(curBFC);
				else ldtab.back()->SetBFC(invertsub);
			}
		}
		ldSubfile.close();
		if(Inlined)
		{
			string line = "Inlined: ";
			line += ldtab[i]->toString();
			delete ldtab[i];
			if(SubComments)	ldtab[i]=new LdComment(line.c_str());
			else ldtab[i]=new LdEmpty;
			if(i>0)
			{
				if((ldtab[i-1]->getType() == Comment) && ((LdComment *)ldtab[i-1])->readbfc() == 1)
				{
					delete ldtab[i-1];
					ldtab[i-1]=new LdEmpty;
				}
			}
		}
	}
	cout << "\nInlined from subfiles:\n";
	cout << numEdge << " Edge lines";
	// if(DelEdge) cout << " (deleted)";
	cout << endl << numCond << " Conditional lines";
	// if(DelCond) cout << " (deleted)";
	cout << endl << numTri << " Triangles"<< endl;
	cout << numQuad << " Quads"<< endl;
	m_nLine = ldtab.size();
}

bool Fldraw::ldSave (char *fname)
{
	int i=0;
	int numline=0;
	int numEdge=0;
	int numCond=0;
	int numTri=0;
	int numQuad=0;
	int numSub=0;
	ofstream ldFileOut(fname, ios::out);
	if(ldFileOut == NULL) return false;
	if(WriteNewOnly) i = m_nLine;
	if(!Inlined || SubComments)
	{
		for(; i<ldtab.size(); i++)
		{
			if(!Inlined)
			{
				if(ldtab[i]->GetNoOut()) continue;
			}
			switch(ldtab[i]->getType())
			{
			case Triangle:
				numTri++;
				break;
			case Quad:
				numQuad++;
				break;
			case Line:
				numEdge++;
				break;
			case Condline:
				numCond++;
				break;
			case Subfile:
				numSub++;
				break;
			}
			if(ldtab[i]->toString().length() > 0)	
			{
				ldFileOut << ldtab[i]->toString().c_str() << endl;
				numline++;
			}
		}
	}
	else
	{
		int newBFC;
		int curBFC=0;
		int invertBFC = 1;

		for(; i<ldtab.size(); i++)
		{
			switch(ldtab[i]->getType())
			{
			case Comment:
				newBFC = ((LdComment*)ldtab[i])->readbfc();
				if(newBFC ==0) break;
				if(curBFC == 0) 
				{
					curBFC = newBFC;
					break;
				}
				if(curBFC != newBFC) invertBFC = -1; else invertBFC = 1;
				delete ldtab[i];
				ldtab[i] = new LdEmpty;
				break;
			case Triangle:
				if(invertBFC == -1)	ldtab[i]->InvertWinding();
				numTri++;
				break;
			case Quad:
				if(invertBFC == -1)	ldtab[i]->InvertWinding();
				numQuad++;
				break;
			case Line:
				numEdge++;
				break;
			case Condline:
				numCond++;
				break;
			case Subfile:
				numSub++;
				break;
			}
			if(ldtab[i]->toString().length() > 0)	
			{
				ldFileOut << ldtab[i]->toString().c_str() << endl;
				numline++;
			}
		}
	}
	cout << endl << "Saving " << fname << endl;
	cout << numline << " lines in file"<< endl;
	cout << numEdge << " Edge lines";
	cout << endl << numCond << " Conditional lines";
	cout << endl << numTri << " Triangles"<< endl;
	cout << numQuad << " Quads"<< endl;
	return true;
}

Fldraw::findOrphan()
{
	int OrphanNumber = 0;
//	cout << "Taille avant " << ldtab.size() << endl;
	ldtab.push_back(new LdComment("// Unmatched edges"));
	for(int i=0; i<m_nLine; i++)
	{
		if(!ldtab[i]->isSurface()) continue;
		LdLine *ldl = (LdLine *)ldtab[i]->getFirstLine();
		do
		{
			bool foundmatch=false;
			for(int j=0; j<m_nLine; j++)
			{
				if(j==i) continue;
				if(ldtab[j]->matchLine(*ldl))
				{
					foundmatch=true;
					break;
				}
			}
			if(!foundmatch)
			{
				OrphanNumber++;
				ldl->SetLineColor(4); // Orphan lines are red
				ldtab.push_back(ldl);
			}
			else delete ldl;

		} while ((ldl = (LdLine *)ldtab[i]->getNextLine())!=NULL);
	
	}
//	cout << "Taille apres " << ldtab.size() << endl;
	cout << endl << OrphanNumber << " Unmatched edges found" << endl;
}

void Fldraw::deleteElement(int i)
{
	if((ldtab[i]->getType()==Subfile) && (i>0))
	{
		if(ldtab[i-1]->getType()==Comment)
		{
			string s=ldtab[i-1]->toString();
			if((s.find("BFC") != string::npos) && (s.find("INVERTNEXT") != string::npos))
			{
				delete ldtab[i-1];
				ldtab[i-1]=new LdEmpty;
			}
		}
	}
	delete ldtab[i];
	ldtab[i]=new LdEmpty;
}

void Fldraw::pushElement(LdObject *ldo)
{
	ldtab.push_back(ldo);
}


void Fldraw::separate(Fldraw *sep)
{
	for (int i=0; i<sep->ldtab.size(); i++)
	{
		if(!sep->ldtab[i]->isSurface()) continue;
		separate(sep->ldtab[i]);
	}
}

void Fldraw::separate(LdObject *ldo)
{
	for (int i=0; i<ldtab.size(); i++)
	{
		int Side=1;
		string s;
		if(ldtab[i]->getType() == Subfile)
		{
			s=ldtab[i]->toString();
			if((s.find("stud.dat") != string::npos) || (s.find("stud2.dat") != string::npos) || (s.find("stug") != string::npos)) Side=0;
		}
		
		if(Side!=0) Side = ldtab[i]->checkSide(ldo);
		if(Side==-1)
		{
			if(!Nothing) deleteElement(i);
		}
		if((Side==1) && Middle)
		{
			if(ldtab[i]->getType() != Comment)  
				if(!Nothing) deleteElement(i);
		}
		else if(Side==0)
		{
			if (Coloring)
			{
				if(ldtab[i]->isLine()) ldtab[i]->SetColor(4);
				if(ldtab[i]->isSurface() || (ldtab[i]->getType() == Subfile)) ldtab[i]->SetColor(14);
			}
			if(Verify)
				if(!ldtab[i]->checkSym(ldo))
				{
					if(ldtab[i]->isLine()) ldtab[i]->SetColor(10);
					if(ldtab[i]->isSurface()) ldtab[i]->SetColor(5);
				}
			if(Across)
			{
				if(!Nothing) deleteElement(i);
			}
			if(CutAcross)
			{
				LdObject *p1, *p2;
				p1 = p2 = NULL;
				ldtab[i]->Cut(ldo,p1,p2);
				delete ldtab[i];
				if(p1!=NULL) ldtab[i]=p1; // p1 = NULL should never occur.
				else ldtab[i] = new LdEmpty;
				if(p2!=NULL) ldtab.push_back(p2);
			}
		}
		else if(Side==2)
		{
			if (Coloring)
			{
				if(ldtab[i]->isLine() || (ldtab[i]->getType() == Subfile)) ldtab[i]->SetColor(1);
				if(ldtab[i]->isSurface()) ldtab[i]->SetColor(9);
			}
			if(InSplit)
			{
				if(!Nothing) deleteElement(i);
			}
		}
	}
}

/*
Fldraw::findAdjacent()
{
	int addLines=0;
	int addCond=0;
	int addInBetween=0;
	int Curcolor;
//	cout << "Taille avant " << ldtab.size() << endl;
	ldtab.push_back(new LdComment("// Added lines/Condlines"));
	for(int i=0; i<m_nLine; i++)
	{
		if(!ldtab[i]->isSurface()) continue;
		LdLine *ldl = (LdLine *)ldtab[i]->getFirstLine();
		do
		{
			bool foundmatch=true;
			for(int j=i+1; j<m_nLine; j++)
			{
				if(ldtab[j]->matchLine(*ldl))
				{
					for(int k=0; k<m_nLine; k++)
					{
						if(!ldtab[k]->isLine()) continue;
						if(ldtab[k]->matchLine(*ldl))
						{
							foundmatch=false;
							break;
						}
					}
					if(foundmatch)
					{
						double angle=ldtab[i]->getAngle(*ldtab[j], (LdObject&)*ldl);
						// cout << endl << angle << " degrees" << endl;
						if(fabs(angle) < LdFlat) 
						{
							delete ldl; 
							break;
						}
						if (fabs(angle) > LdMinEdge)
						{
							// Added lines are pink or edge color
							if(DebugColors) Curcolor=13; else Curcolor = 24;
							addLines++;
							ldl->SetLineColor(Curcolor); 
							ldtab.push_back(ldl);
							break;
						}
						if (fabs(angle) < LdMaxCond)
						{
							// Added condlines are light blue or edge color
							if(angle * CxCv > 0) break; 
							if(DebugColors) Curcolor=9; else Curcolor = 24;
							addCond++;
							ldtab.push_back(new LdCondline(*(ldl->getP1()), *(ldl->getP2()), 
								*(ldtab[i]->getOtherPoint(*ldl)), *(ldtab[j]->getOtherPoint(*ldl)), Curcolor));
							delete ldl;
							break;
						}

						if(angle * CxCv > 0) break; 
						addInBetween++;
						ldl->SetLineColor(2); // Added in-between lines are bright green
						ldtab.push_back(ldl);
						ldtab.push_back(new LdCondline(*(ldl->getP1()), *(ldl->getP2()), 
							*(ldtab[i]->getOtherPoint(*ldl)), *(ldtab[j]->getOtherPoint(*ldl)), 2)); // Added condlines are green
						break;
					}
				}
			}
			if(!foundmatch) delete ldl;
		} while ((ldl = (LdLine *)ldtab[i]->getNextLine())!=NULL);
	}
//	cout << "Taille apres " << ldtab.size() << endl;
	cout << endl << addLines << " New edges found" << endl;
	cout << addCond << " New conditional lines found" << endl;
	cout << addInBetween << " New dubious lines/condlines found" << endl;
}

*/

